Package: snakecase
Version: 0.11.1
Date: 2023-08-27
Title: Convert Strings into any Case
Description: A consistent, flexible and easy to use tool to parse and convert strings into cases like snake or camel among others.
Authors@R: c(
    person("Malte", "Grosser", , "malte.grosser@gmail.com", role = c("aut", "cre")))
Maintainer: Malte Grosser <malte.grosser@gmail.com>
Depends: R (>= 3.2)
Imports: stringr, stringi
Suggests: testthat, covr, tibble, purrrlyr, knitr, rmarkdown, magrittr
URL: https://github.com/Tazinho/snakecase
BugReports: https://github.com/Tazinho/snakecase/issues
Encoding: UTF-8
License: GPL-3
RoxygenNote: 6.1.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-08-27 20:30:20 UTC; malte
Author: Malte Grosser [aut, cre]
Repository: CRAN
Date/Publication: 2023-08-27 22:50:09 UTC
Built: R 4.6.0; ; 2026-04-27 21:49:57 UTC; unix
